//
//  PhysicalSimulator.h
//  BouncingBallWithOde
//

#ifndef __Test__PhysicalSimulator__
#define __Test__PhysicalSimulator__

#include <boost/noncopyable.hpp>
#include <boost/shared_ptr.hpp>
#include <ode/ode.h>
#include <ode/odecpp_collision.h>
#include <ode/odecpp.h>
#include <vector>
#include <pcl/surface/gp3.h>
#include "Device.h"

class PhysicalSimulator : private boost::noncopyable
{
public:
        // A ball consists of a rigid body and a geometry.
        struct Ball
        {
                dBody   body_;
                dSphere geom_;
        };

private:
        dWorld          world_;
        dHashSpace      space_;
        dJointGroup     joint_group_;
        
        std::vector<boost::shared_ptr<Ball> > balls_;
        
        dGeomID triangular_mesh_;
        
        // the maximum number of contact points
        const static int MaxNumOfContacts = 1;
        std::vector<dContact> contacts_;
        
        bool collides_;
        
public:
        PhysicalSimulator();
        ~PhysicalSimulator();
        
        const std::vector<boost::shared_ptr<Ball> >& get_balls() const
        {
                return balls_;
        }
        
        void create_world();
        void near_callback(void* data, dGeomID o1, dGeomID o2);
        void collide(void(*near_callback)(void* data, dGeomID o1, dGeomID o2));
        void create_balls();
        bool collides() const
        {
                return collides_;
        }
        
        void set_collides(bool flag)
        {
                collides_ = flag;
        }
        
        void empty_join_group()
        {
                joint_group_.empty();
        }
        
        void step(dReal step)
        {
                world_.step(step);
        }
        
        void register_triangular_mesh(const boost::shared_ptr<Device::TriangularMesh>& mesh);
        
        void clear_balls();
        
};
#endif /* defined(__Test__PhysicalSimulator__) */
